const deepMerge = require('deepmerge')
const formsPlugin = require('@tailwindcss/forms')
const { colors, fontFamily } = require('tailwindcss/defaultTheme')

const uiConfig = {
  purge: {
    content: [
    'node_modules/@mondra/ui-components/dist/index.js',
    './src/**/*.html',
    './src/**/*.jsx',
    './src/**/*.tsx',
    './src/**/*.ts',
  ]},
  theme: {
    fontFamily: {
      inter: ['Inter', ...fontFamily.sans],
    },
    colors: {
      primary: {
        50: '#F0FAFA',
        100: '#D5F1F1',
        200: '#B6E2E2',
        300: '#94D0D0',
        400: '#6EB7B7',
        500: '#4B9B9B',
        600: '#368282',
        700: '#256565',
        800: '#164646',
        900: '#092525',
      },
      secondary: {
        50: '#F0F4F8',
        100: '#F0F4F8',
        200: '#BCCCDC',
        300: '#9FB3C8',
        400: '#829AB1',
        500: '#627D98',
        600: '#486581',
        700: '#334E68',
        800: '#243C53',
        900: '#192C3D',
      },
      pageBgColor: '#F3F4F6',
      ...colors,
    },
    container: {
      padding: '2rem',
    },
    extend: {
      spacing: {
        14: '3.5rem',
      },
      boxShadow: {
        focus:
          '#ffffff 0px 0px 0px 0px, #6366f1 0px 0px 0px 2px, rgba(0, 0, 0, 0.05) 0px 1px 2px 0px',
      },
      gridColumn: {
        'span-13': 'span 13 / span 13',
        'span-14': 'span 14 / span 14',
        'span-15': 'span 15 / span 15',
        'span-16': 'span 16 / span 16',
        'span-17': 'span 17 / span 17',
        'span-18': 'span 18 / span 18',
        'span-19': 'span 19 / span 19',
        'span-20': 'span 20/ span 20',
      },
      height: {
        'app-header': '56px',
        'brand-logo': '150px',
        'card-header': '56px',
      },
      opacity: {
        80: '0.8',
      },
      width: {
        'brand-logo': '180px',
        'sidebar-close': '80px',
        'sidebar-open': '256px',
        31: '6rem',
      },
      minWidth: {
        12: '48px',
        'sidebar-close': '80px',
        'sidebar-open': '256px',
      },
      maxWidth: {
        'company-selection': '150px',
        'sidebar-close': '80px',
        'sidebar-open': '256px',
      },
      minHeight: {
        button: '32px',
        'scroll-height': 'calc(100vh - 56px)',
      },
      maxHeight: {
        'scroll-height': 'calc(100vh - 56px)',
      },
      screens: {
        xxl: { min: '1600px' },
      },
      keyframes: {
        'slide-down': {
          '0%': {
            opacity: 0,
            transform: 'translateY(-10px)',
          },
          '100%': { opacity: 1, transform: 'translateY(0)' },
        },
        'fade-in': {
          '0%': {
            opacity: 0,
          },
          '100%': { opacity: 1 },
        },
      },
      animation: {
        'slide-down': 'slide-down 150ms ease-in',
        'fade-in': 'fade-in 200ms ease-in-out',
      },
      zIndex: {
        '-1': -1
      }
    },
  },
  variants: {
    backgroundOpacity: ['responsive', 'hover', 'focus'],
    backgroundColor: ['responsive', 'hover', 'focus', 'active', 'odd', 'disabled', 'checked', 'focus-within', 'group-hover'],
    display: ['responsive', 'dark'],
    textColor: ['responsive', 'focus', 'focus-within', 'hover', 'active', 'disabled', 'checked', 'group-hover'],
    placeholderColor: ['responsive', 'focus'],
    borderColor: ['responsive', 'hover', 'focus', 'checked'],
    divideColor: ['responsive'],
    boxShadow: ['responsive', 'hover', 'focus'],
    margin: ['responsive', 'last'],
    opacity: ['disabled']
  },
  plugins: [formsPlugin],
}

function arrayMergeFn(destinationArray, sourceArray) {
  return destinationArray.concat(sourceArray).reduce((acc, cur) => {
    if (acc.includes(cur)) return acc
    return [...acc, cur]
  }, [])
}

/**
 * Merge UI components config and Tailwind CSS configurations
 * @param {object} tailwindConfig - Tailwind config object
 * @return {object} new config object
 */
function wrapper(tailwindConfig) {
  let purge
  if (Array.isArray(tailwindConfig.purge)) {
    purge = {
      content: tailwindConfig.purge,
    }
  } else {
    purge = tailwindConfig.purge
  }
  return deepMerge({ ...tailwindConfig, purge }, uiConfig, {
    arrayMerge: arrayMergeFn,
  })
}

module.exports = wrapper
